/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema, Romain Lenglet
 */

package org.objectweb.dream.aggregator;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.objectweb.dream.Pull;
import org.objectweb.dream.PullException;
import org.objectweb.dream.dreamannotation.DreamComponent;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManagerType;
import org.objectweb.fractal.fraclet.annotation.annotations.type.Cardinality;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;

/**
 * This aggregator has a collection of pull inputs, and a pull output. When a
 * message is pulled on the output, the aggregator pulls messages on its input,
 * aggregates them, and returns the aggregated message. Messages are pulled
 * sequentially in a non predictable order. Input pull interfaces can return
 * <code>null</code>.
 */
@DreamComponent(controllerDesc = "dreamPrimitive")
@Provides(interfaces=@Interface(name = "out-pull", signature=Pull.class))
public class PullPullAggregatorCollectionInput
    implements
      Pull
{

  @Requires(name = "in-pull", cardinality = Cardinality.COLLECTION)
  private Map<String, Pull> inPullItfs = new HashMap<String, Pull>();

  /** the message manager client interface of this component */
  @Requires(name = "message-manager")
  protected MessageManagerType  messageManagerItf;

  /**
   * @see Pull#pull()
   */
  public Message pull() throws PullException
  {
    Message outMessage = messageManagerItf.createMessage();
    Iterator<Pull> iter = inPullItfs.values().iterator();
    while (iter.hasNext())
    {
      Pull pullItf = iter.next();
      Message msg = pullItf.pull();
      if (msg != null)
      {
        messageManagerItf.addSubMessage(outMessage, msg);
      }
    }
    return outMessage;
  }
}
