/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Romain Lenglet
 */

package org.objectweb.dream.exception;

import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.dreamannotation.DreamComponent;
import org.objectweb.dream.dreamannotation.DreamMonolog;
import org.objectweb.dream.message.Message;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.Service;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * Try to push incoming messages on the output (<code>"out-push"</code>
 * client interface). If a {@link PushException} is catched, the message is
 * pushed on the <code>"exception-push"</code> client interface.
 */
@DreamComponent(controllerDesc = "dreamPrimitive")
@Provides(interfaces={@Interface(name="in-push", signature=Push.class)})
public class CatchPushException implements Push
{
  // ---------------------------------------------------------------------------
  // Client interfaces
  // ---------------------------------------------------------------------------
  
  @Requires(name = "out-push")
  protected Push             outPushItf;
  
  /**
   * The default name of the out push interface used if a {@link PushException}
   * has been catched while a message was pushed on the normal out push.
   */
  public static final String EXCEPTION_PUSH_ITF_NAME = "exception-push";
  @Requires(name = EXCEPTION_PUSH_ITF_NAME)
  protected Push             exceptionPushItf;

  /**
   * Component reference
   */
  @Service
  Component weaveableC;
  
  /**
   * Logger "impl"
   */ 
  @DreamMonolog()
  protected Logger                   logger;
  
  // ---------------------------------------------------------------------------
  // Implementation of the Push interface
  // ---------------------------------------------------------------------------

  /**
   * @see Push#push(Message)
   */
  public void push(Message message) throws PushException
  {
    try
    {
      outPushItf.push(message);
    }
    catch (PushException e)
    {
      logger.log(BasicLevel.INFO,
          "Catched Push Exception push message on exception-push. ", e);
      exceptionPushItf.push(message);
    }
  }
}