/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: mohamed.mokhtari@insa-lyon.fr
 *
 * Initial developer(s): Mohamed Mokhtari
 */

package org.objectweb.dream.protocol;

import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

import org.objectweb.dream.message.AbstractChunk;

/**
 * A chunk containing a protocol
 */
public class ProtocolChunk extends AbstractChunk<ProtocolChunk> {
  //private static final long  serialVersionUID = 3257570624300986424L;

  /** The default name of chunk of this type */
  public static final String DEFAULT_NAME     = "protocol-chunk";

  private String protocol = "tcpip";

  /**
   * Returns the protocol.
   * 
   * @return the protocol.
   */
  public final String getProtocol() {
    return protocol;
  }

  /**
   * Set the protocol.
   * 
   * @param message the protocol.
   */
  public void setProtocol(String protocol) {
    this.protocol = protocol;
  }

  /**
   * @see AbstractChunk#newChunk()
   */
  public ProtocolChunk newChunk() {
    return new ProtocolChunk();
  }

  /**
   * @see AbstractChunk#transfertStateTo(AbstractChunk)
   */
  public void transfertStateTo(ProtocolChunk newInstance) {
    newInstance.setProtocol(protocol);
  }

  /**
   * @see org.objectweb.dream.pool.Recyclable#recycle()
   */
  public void recycle() {
    protocol = null;
  }

  /**
   * @see java.io.Externalizable#readExternal(java.io.ObjectInput)
   */
  public void readExternal(ObjectInput in) throws IOException,
      ClassNotFoundException {
    protocol = in.readUTF();
  }

  /**
   * @see java.io.Externalizable#writeExternal(java.io.ObjectOutput)
   */
  public void writeExternal(ObjectOutput out) throws IOException {
    out.writeUTF(protocol);
  }
}