/**
 * DREAM
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.pushwithreturn;

import java.util.Iterator;

import org.objectweb.fractal.fraclet.annotation.annotations.Interface;

/**
 * This component manages {@link WaitingKey}objects. It maps them to
 * {@link Key}objects.
 */
@Interface(name = WaitingKeysItf.ITF_NAME)
public interface WaitingKeysItf
{
  /**
   * The commonly used name to refer to this interface.
   */
  String ITF_NAME = "waiting-keys";

  /**
   * Returns the {@link WaitingKey}object corresponding to the given key.
   * 
   * @param key the key that maps the returned WaitingKey object.
   * @return a WaitingKey object if one maps the given key, <code>null</code>
   *         otherwise.
   * @throws InterruptedException if an exception occurs.
   */
  WaitingKey removeWaitingKey(Key key) throws InterruptedException;

  /**
   * Returns an iterator over {@link WaitingKey}objects corresponding to the
   * keys that are associated to the given key (that is an "isAll" key).
   * 
   * @param key the "isAll" key that maps the returned keys.
   * @return an iterator over WaitingKey objects.
   * @throws InterruptedException if an exception occurs.
   */
  Iterator removeWaitingKeys(Key key) throws InterruptedException;

  /**
   * Creates a {@link WaitingKey}object for the given key.
   * 
   * @param key the key for which a WaitingKey object must be created.
   * @return the created WaitingKey object.
   * @throws InterruptedException if an exception occurs.
   */
  WaitingKey createWaitingKey(Key key) throws InterruptedException;
}
