/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Romain Lenglet
 * Contributor(s):
 */

package org.objectweb.dream.queue;

import org.objectweb.dream.message.Message;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;

/**
 * Determines which messages in a buffer can be returned by calls to
 * {@link org.objectweb.dream.queue.buffer.BufferRemove#get()} and
 * {@link org.objectweb.dream.queue.buffer.BufferRemove#remove()}. The
 * criterion that is implemented by a matcher to select messages can change
 * dynamically, i.e. different calls to {@link #acceptMessage(Message)} with the
 * same message specified as a parameter can return different results.
 * Implementations of the {@link #acceptMessage(Message)} should not be
 * blocking, and should be as efficient as possible, since this method is often
 * called by buffers.
 */
@Interface(name = MessageMatcher.ITF_NAME)
public interface MessageMatcher
{

  /**
   * The commonly used name to refer to an interface with this signature.
   */
  String ITF_NAME = "message-matcher";

  /**
   * Tests if the specified message can be returned from a buffer.
   * 
   * @param message the message to test; must not be <code>null</code>.
   * @return true only if the message can be returned.
   */
  boolean acceptMessage(Message message);

}