/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Romain Lenglet
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.queue.handler.incoming;

import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.dreamannotation.DreamComponent;

/**
 * An incoming handler that adds {@link #push(Message) pushed} messages into a
 * {@link org.objectweb.dream.queue.buffer.BufferAdd buffer} bound to it. This component uses
 * {@link org.objectweb.dream.queue.buffer.BufferAdd#tryAdd(Message) non blocking calls for trying to add}
 * messages into the buffer: if a message cannot be added, it is deleted
 * ("dropped").
 */
@DreamComponent(controllerDesc = "dreamUnstoppablePrimitive")
public class IncomingHandlerDropping extends AbstractIncomingHandler
{

  // ------------------------------------------------------------------------
  // Implementation of the Push interface.
  // ------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.Push#push(Message)
   */
  public void push(Message message) throws PushException
  {
    if (!bufferAdd.tryAdd(message))
    {
      messageManagerItf.deleteMessage(message);
    }
  }
}