/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Romain Lenglet
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.queue.handler.outgoing;

import org.objectweb.dream.PullException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.dreamannotation.DreamComponent;

/**
 * An outgoing handler that {@link #pull() returns} messages removed from a
 * {@link org.objectweb.dream.queue.buffer.BufferRemove buffer} bound to it.
 * This component uses
 * {@link org.objectweb.dream.queue.buffer.BufferRemove#removeAll() non blocking calls for removing}
 * ALL messages from the buffer. The pulled messages are aggregated messages
 * that contain as sub messages all the messages that have been removed from the
 * buffer. If no message is removed from the buffer, the returned message is
 * <code>null</code>
 */
@DreamComponent(controllerDesc = "dreamUnstoppablePrimitive")
public class OutgoingHandlerAggregating extends AbstractOutgoingHandler
{

  // ------------------------------------------------------------------------
  // Implementation of the Pull interface.
  // ------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.Pull#pull()
   */
  public Message pull() throws PullException
  {
    return bufferRemove.removeAll();
  }
}
