/**
 * DREAM
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.serializator;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;

import org.objectweb.dream.message.codec.ByteArrayCodecInputOutput;

/**
 * An CodecInputOutput that contains a ByteArrayOutputStream.
 */
public class CodecInputOutputByteArray extends ByteArrayCodecInputOutput
{

  protected ByteArrayOutputStream byteArrayOutputStream;

  /**
   * Returns the byte array associated to this codec.
   * 
   * @return the byte array associated to this codec.
   */
  public byte[] getByteArray()
  {
    return byteArrayOutputStream.toByteArray();
  }

  /**
   * @see org.objectweb.dream.message.codec.CodecInputOutput#getOutput()
   */
  public Object getOutput() throws IOException
  {
    if (byteArrayOutputStream == null)
    {
      byteArrayOutputStream = new ByteArrayOutputStream();
      output = byteArrayOutputStream;
    }
    return output;
  }

  /**
   * @see org.objectweb.dream.pool.Recyclable#recycle()
   */
  public void recycle()
  {
    input = null;
    if (byteArrayOutputStream != null)
    {
      byteArrayOutputStream.reset();
      if (output instanceof ObjectOutputStream)
      {
        try
        {
          ((ObjectOutputStream) output).reset();
        }
        catch (IOException e)
        {
          output = null;
        }
      }
    }
  }

}