/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.serializator;

import java.io.IOException;

import org.objectweb.dream.message.Message;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;

/**
 * A serializator defines a method <code>serialize</code> that takes a message
 * as parameter and returns representation of the message with the given generic
 * type (example : byte array).
 */
@Interface(name = SerializatorItf.ITF_NAME)
public interface SerializatorItf<T>
{

  /** The default name of the {@link SerializatorItf }interface. */
  String ITF_NAME = "serializator";

  /**
   * Serializes the given message and returns a representation of the message
   * with the given generic type (example : byte array).
   * 
   * @param message the message to be serialized.
   * @return A representation of the message with the given generic type
   *         (example : byte array).
   * @throws IOException if an exception occurs while serializing the message.
   */
  T serialize(Message message) throws IOException;
}