/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): Romain Lenglet
 */

package org.objectweb.dream.synchro;


import org.objectweb.dream.InterruptedPushException;
import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.dreamannotation.DreamComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;

/**
 * This component has a Push input. When it receives a message on its input, it
 * locks a mutex (to which it is bound), forwards the message on its Push
 * output, and then unlocks the mutex.
 */
@DreamComponent(controllerDesc = "dreamPrimitive")
@Provides(interfaces=@Interface(name = "in-push", signature = Push.class))
public class LockPushUnlock  implements Push
{
  @Requires(name = "out-push")
  protected Push  outPushItf;
  @Requires(name = MutexItf.ITF_NAME)
  protected MutexItf mutexItf;

  // ---------------------------------------------------------------------------
  // Implementation of the Push interface.
  // ---------------------------------------------------------------------------

  /**
   * @see Push#push(Message)
   */
  public void push(Message message) throws PushException
  {
    try
    {
      mutexItf.lock();
    }
    catch (InterruptedException e)
    {
      throw new InterruptedPushException(e);
    }
    try
    {
      outPushItf.push(message);
    }
    finally
    {
      mutexItf.unlock();
    }
  }
}