/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): Romain Lenglet
 */

package org.objectweb.dream.trash;


import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManagerType;
import org.objectweb.dream.dreamannotation.DreamComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;

/**
 * This component is a trash: received messages are deleted.
 */
@DreamComponent(controllerDesc = "dreamUnstoppablePrimitive")
@Provides(interfaces=@Interface(name="",signature=Push.class))
public class Trash 
    implements
      Push,
      TrashMonitoring
{

  // ---------------------------------------------------------------------------
  // Client interface.
  // ---------------------------------------------------------------------------
  @Requires(name = "message-manager") 
  protected MessageManagerType messageManagerItf;

  // ---------------------------------------------------------------------------
  // Fields.
  // ---------------------------------------------------------------------------
  private int              nbDeletedMessages;

  // ---------------------------------------------------------------------------
  // Implementation of the Push interface.
  // ---------------------------------------------------------------------------

  /**
   * @see Push#push(Message)
   */
  public void push(Message message) throws PushException
  {
    nbDeletedMessages++;
    messageManagerItf.deleteMessage(message);
  }

  // ---------------------------------------------------------------------------
  // Implementation of the TrashMonitoring interface.
  // ---------------------------------------------------------------------------

  /**
   * @see TrashMonitoring#getNumberOfDeletedMessages()
   */
  public int getNumberOfDeletedMessages()
  {
    return nbDeletedMessages;
  }

  /**
   * @see TrashMonitoring#reset()
   */
  public int reset()
  {
    int old;
    synchronized (this)
    {
      old = nbDeletedMessages;
      nbDeletedMessages = 0;
    }
    return old;
  }

}